import * as i0 from '@angular/core';
import { Injectable, EventEmitter, signal, computed, Component, ChangeDetectionStrategy, Input, Output, NgModule } from '@angular/core';
import { Subject, filter, takeWhile, asyncScheduler, timer } from 'rxjs';
import { style, animate, transition, trigger, animateChild, query } from '@angular/animations';
import { takeUntilDestroyed } from '@angular/core/rxjs-interop';
import * as i1 from '@angular/common';
import { NgForOf, NgIf, NgClass, NgStyle } from '@angular/common';

class Toast {
    static { this.DEFAULT_OPTIONS = {
        hostName: 'default',
        severity: 'plain',
    }; }
    constructor(message, options) {
        this.message = message;
        this.options = {
            ...Toast.DEFAULT_OPTIONS,
            ...options
        };
    }
}
class ToastItemCloseEvent {
    constructor(toast, index) {
        this.toast = toast;
        this.index = index;
    }
}

class SpToastService {
    constructor() {
        this.publisher$ = new Subject();
    }
    show(toast) {
        this.publisher$.next(toast);
    }
    observe(hostName = Toast.DEFAULT_OPTIONS.hostName) {
        return this.publisher$.pipe(filter((toast) => toast != null && hostName === toast.options.hostName));
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastService, deps: [], target: i0.ɵɵFactoryTarget.Injectable }); }
    static { this.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastService, providedIn: 'root' }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastService, decorators: [{
            type: Injectable,
            args: [{
                    providedIn: 'root'
                }]
        }], ctorParameters: () => [] });

const styleFadeNotVisible = style({ opacity: 0, transform: 'translateY(50px)' });
const styleFadeVisible = style({ opacity: 1, transform: 'translateY(0)' });
const fadeAnimationTiming = '0.25s ease';
function toastFadeAnimation() {
    return [
        trigger('toastFade', [
            transition(':enter', [
                styleFadeNotVisible,
                animate(fadeAnimationTiming, styleFadeVisible)
            ]),
            transition(':leave', [
                styleFadeVisible,
                animate(fadeAnimationTiming, styleFadeNotVisible)
            ])
        ])
    ];
}

class SpToastComponent {
    set index(index) {
        this.index$.set(index);
    }
    set toast(toast) {
        this.toast$.set(toast);
        this.life$.set(this.lifeTimeMillis$());
    }
    constructor(zone) {
        this.zone = zone;
        this.close = new EventEmitter();
        this.index$ = signal(null);
        this.toast$ = signal(null);
        this.life$ = signal(3000);
        this.lifeTimeMillis$ = computed(() => this.toast$()?.options?.lifeMillis ?? 3000);
        this.progressPercent$ = computed(() => 100 - (this.life$() / this.lifeTimeMillis$() * 100));
        this.toastStyleClasses$ = computed(() => {
            const classes = [];
            const severity = this.toast$()?.options?.severity;
            if (severity != null) {
                classes.push(`toast-${severity}`);
            }
            return classes;
        });
        this.closeButtonStyleClasses$ = computed(() => {
            const classes = [];
            const severity = this.toast$()?.options?.severity;
            if (severity != null && severity !== 'plain') {
                classes.push(`button-${severity}`);
            }
            else if (severity === 'plain') {
                classes.push(`button-secondary`);
            }
            return classes;
        });
        this.lifeIntervalMillis = 500;
        this.lifeIndicatorStyles$ = computed(() => {
            const absolutePosition = this.progressPercent$() / 2;
            return {
                left: `${absolutePosition}%`,
                right: `${absolutePosition}%`,
                transitionDuration: `${this.lifeIntervalMillis}ms`
            };
        });
        timer(0, this.lifeIntervalMillis, asyncScheduler).pipe(takeUntilDestroyed(), takeWhile(() => this.life$() > 0)).subscribe(() => {
            this.life$.update(life => life -= this.lifeIntervalMillis);
        });
    }
    ngAfterViewInit() {
        this.initTimeout();
    }
    ngOnDestroy() {
        this.clearTimeout();
    }
    onCloseClicked() {
        this.zone.run(() => this.close.emit(new ToastItemCloseEvent(this.toast$(), this.index$())));
    }
    initTimeout() {
        this.zone.runOutsideAngular(() => {
            this.timeout = setTimeout(() => this.onCloseClicked(), this.lifeTimeMillis$());
        });
    }
    clearTimeout() {
        if (this.timeout) {
            clearTimeout(this.timeout);
            this.timeout = null;
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastComponent, deps: [{ token: i0.NgZone }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "17.0.8", type: SpToastComponent, selector: "sp-toast", inputs: { index: "index", toast: "toast" }, outputs: { close: "close" }, ngImport: i0, template: "<div class=\"sp-toast\" [ngClass]=\"toastStyleClasses$()\" @toastFade>\n    <div class=\"toast-life-indicator\" [ngStyle]=\"lifeIndicatorStyles$()\"></div>\n    <div class=\"sp-toast-content\">\n        {{ toast$()?.message }}\n    </div>\n    <button class=\"toast-button-close sp-button fab button-text\" [ngClass]=\"closeButtonStyleClasses$()\" (click)=\"onCloseClicked()\">\n        <svg viewBox=\"0 0 26 26\" class=\"sp-button-icon\" style=\"fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2\">\n            <path d=\"M24.042 1.414a2 2 0 0 0-2.828 0l-19.8 19.8a2 2 0 0 0 2.829 2.828l19.799-19.8a2 2 0 0 0 0-2.828z\"/>\n            <path d=\"M1.414 1.414a2 2 0 0 0 0 2.828l19.8 19.8a2 2 0 0 0 2.828-2.829L4.24 1.413a1.998 1.998 0 0 0-2.825 0l-.002.001z\"/>\n        </svg>\n    </button>\n</div>\n", styles: [""], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "directive", type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }], animations: [toastFadeAnimation()], changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-toast', animations: [toastFadeAnimation()], changeDetection: ChangeDetectionStrategy.OnPush, template: "<div class=\"sp-toast\" [ngClass]=\"toastStyleClasses$()\" @toastFade>\n    <div class=\"toast-life-indicator\" [ngStyle]=\"lifeIndicatorStyles$()\"></div>\n    <div class=\"sp-toast-content\">\n        {{ toast$()?.message }}\n    </div>\n    <button class=\"toast-button-close sp-button fab button-text\" [ngClass]=\"closeButtonStyleClasses$()\" (click)=\"onCloseClicked()\">\n        <svg viewBox=\"0 0 26 26\" class=\"sp-button-icon\" style=\"fill-rule:evenodd;clip-rule:evenodd;stroke-linejoin:round;stroke-miterlimit:2\">\n            <path d=\"M24.042 1.414a2 2 0 0 0-2.828 0l-19.8 19.8a2 2 0 0 0 2.829 2.828l19.799-19.8a2 2 0 0 0 0-2.828z\"/>\n            <path d=\"M1.414 1.414a2 2 0 0 0 0 2.828l19.8 19.8a2 2 0 0 0 2.828-2.829L4.24 1.413a1.998 1.998 0 0 0-2.825 0l-.002.001z\"/>\n        </svg>\n    </button>\n</div>\n" }]
        }], ctorParameters: () => [{ type: i0.NgZone }], propDecorators: { index: [{
                type: Input
            }], toast: [{
                type: Input
            }], close: [{
                type: Output
            }] } });

function toastHostAnimation() {
    return [
        trigger('toastAnimation', [
            transition(':enter, :leave', [
                query('@*', animateChild(), { optional: true })
            ])
        ]),
    ];
}
const styleQueueCounterVisible = style({ opacity: 1, transform: 'scale(1)' });
const styleQueueCounterNotVisible = style({ opacity: 0, transform: 'scale(0)' });
const queueCounterAnimationTiming = '0.1s ease-out';
function toastQueueCounterAnimation() {
    return [
        trigger('queueCounterAnimation', [
            transition(':enter', [
                styleQueueCounterNotVisible,
                animate(queueCounterAnimationTiming, styleQueueCounterVisible)
            ]),
            transition(':leave', [
                styleQueueCounterVisible,
                animate(queueCounterAnimationTiming, styleQueueCounterNotVisible)
            ])
        ])
    ];
}

class SpToastHostComponent {
    set hostName(hostName) {
        this.hostName$.set(hostName);
    }
    set preventDuplicates(preventDuplicates) {
        this.preventDuplicates$.set(preventDuplicates);
    }
    set posVertical(posVertical) {
        this.posVertical$.set(posVertical);
    }
    set posHorizontal(posHorizontal) {
        this.posHorizontal$.set(posHorizontal);
    }
    constructor(toastService) {
        this.toastService = toastService;
        this.hostName$ = signal(Toast.DEFAULT_OPTIONS.hostName);
        this.showMultiple$ = signal(false);
        this.preventDuplicates$ = signal(false);
        this.posVertical$ = signal('top');
        this.posHorizontal$ = signal(null);
        this.hostStyleClasses$ = computed(() => {
            const classes = [`${this.posVertical$()}`];
            if (this.posHorizontal$() != null) {
                classes.push(`${this.posHorizontal$()}`);
            }
            return classes;
        });
        this.toasts$ = signal([]);
        this.nextToast$ = computed(() => {
            if (this.toasts$().length <= 0) {
                return [];
            }
            if (this.showMultiple$()) {
                return [...this.toasts$().slice(0, 5)];
            }
            return [this.toasts$()[0]];
        });
    }
    ngOnInit() {
        this.observeToasts(this.toastService);
    }
    observeToasts(toastService) {
        toastService.observe(this.hostName$())
            .subscribe(toast => this.addToast(toast));
    }
    addToast(toast) {
        if (toast == null) {
            return;
        }
        if (!this.canAddToast(toast)) {
            return;
        }
        this.toasts$.update(existingToasts => [...existingToasts, toast]);
    }
    canAddToast(toast) {
        if (toast.options.id != null && this.preventDuplicates$()) {
            const toastAlreadyExists = this.toasts$().find(existingToast => existingToast.options.id === toast.options.id) != null;
            return !toastAlreadyExists;
        }
        return true;
    }
    removeToast(event) {
        this.toasts$.update(existingToasts => {
            existingToasts.splice(event.index, 1);
            return [
                ...existingToasts
            ];
        });
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastHostComponent, deps: [{ token: SpToastService }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "17.0.0", version: "17.0.8", type: SpToastHostComponent, selector: "sp-toast-host", inputs: { hostName: "hostName", preventDuplicates: "preventDuplicates", posVertical: "posVertical", posHorizontal: "posHorizontal" }, ngImport: i0, template: "<div class=\"sp-toast-host\" [ngClass]=\"hostStyleClasses$()\">\r\n    @if (!showMultiple$() && toasts$().length > 1) {\r\n        <div class=\"sp-toast-queue-counter\"\r\n             @queueCounterAnimation\r\n        >\r\n            {{ toasts$().length - 1}}\r\n        </div>\r\n    }\r\n\r\n    @for (toast of nextToast$(); track toast) {\r\n        <sp-toast [toast]=\"toast\"\r\n                  [index]=\"$index\"\r\n                  (close)=\"removeToast($event)\"\r\n                  @toastAnimation\r\n        ></sp-toast>\r\n    }\r\n</div>\r\n", styles: [""], dependencies: [{ kind: "directive", type: i1.NgClass, selector: "[ngClass]", inputs: ["class", "ngClass"] }, { kind: "component", type: SpToastComponent, selector: "sp-toast", inputs: ["index", "toast"], outputs: ["close"] }], animations: [
            toastHostAnimation(),
            toastQueueCounterAnimation()
        ] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: SpToastHostComponent, decorators: [{
            type: Component,
            args: [{ selector: 'sp-toast-host', animations: [
                        toastHostAnimation(),
                        toastQueueCounterAnimation()
                    ], template: "<div class=\"sp-toast-host\" [ngClass]=\"hostStyleClasses$()\">\r\n    @if (!showMultiple$() && toasts$().length > 1) {\r\n        <div class=\"sp-toast-queue-counter\"\r\n             @queueCounterAnimation\r\n        >\r\n            {{ toasts$().length - 1}}\r\n        </div>\r\n    }\r\n\r\n    @for (toast of nextToast$(); track toast) {\r\n        <sp-toast [toast]=\"toast\"\r\n                  [index]=\"$index\"\r\n                  (close)=\"removeToast($event)\"\r\n                  @toastAnimation\r\n        ></sp-toast>\r\n    }\r\n</div>\r\n" }]
        }], ctorParameters: () => [{ type: SpToastService }], propDecorators: { hostName: [{
                type: Input
            }], preventDuplicates: [{
                type: Input
            }], posVertical: [{
                type: Input
            }], posHorizontal: [{
                type: Input
            }] } });

class ToastModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ToastModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "17.0.8", ngImport: i0, type: ToastModule, declarations: [SpToastComponent,
            SpToastHostComponent], imports: [NgForOf,
            NgIf,
            NgClass,
            NgStyle], exports: [SpToastComponent,
            SpToastHostComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ToastModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "17.0.8", ngImport: i0, type: ToastModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [
                        SpToastComponent,
                        SpToastHostComponent
                    ],
                    imports: [
                        NgForOf,
                        NgIf,
                        NgClass,
                        NgStyle
                    ],
                    exports: [
                        SpToastComponent,
                        SpToastHostComponent
                    ]
                }]
        }] });

/*
 * Public API Surface of toast
 */

/**
 * Generated bundle index. Do not edit.
 */

export { SpToastComponent, SpToastHostComponent, SpToastService, Toast, ToastItemCloseEvent, ToastModule };
//# sourceMappingURL=spicysources-toast.mjs.map
